@@ -443,7 +443,8 @@ int check_auth_arg(char *arg)
             || strcmp(arg, "external") == 0
             || strcmp(arg, "login") == 0
             || strcmp(arg, "ntlm") == 0
-            || strcmp(arg, "oauthbearer") == 0)
+            || strcmp(arg, "oauthbearer") == 0
+            || strcmp(arg, "xoauth2") == 0)
     {
         l = strlen(arg);
         for (i = 0; i < l; i++)
@@ -847,12 +848,13 @@ int check_account(account_t *acc, int sendmail_mode, char **errstr)
 int get_password_eval(const char *arg, char **buf, char **errstr)
 {
     FILE *eval;
-    size_t l;
-    int have_more_data;
+    size_t bufsize;
+    size_t len;
 
     *buf = NULL;
     *errstr = NULL;
     errno = 0;
+    bufsize = 1; /* Account for the null character. */
 
     if (!(eval = popen(arg, "r")))
     {
@@ -864,36 +866,34 @@ int get_password_eval(const char *arg, char **buf, char **errstr)
         return CONF_EIO;
     }
 
-    *buf = xmalloc(LINEBUFSIZE);
-    if (!fgets(*buf, LINEBUFSIZE, eval))
+    do
     {
-        *errstr = xasprintf(_("cannot read output of '%s'"), arg);
-        pclose(eval);
-        free(*buf);
-        *buf = NULL;
-        return CONF_EIO;
-    }
-    have_more_data = (fgetc(eval) != EOF);
-    pclose(eval);
-
-    l = strlen(*buf);
-    if (l > 0)
-    {
-        if ((*buf)[l - 1] != '\n' && have_more_data)
+        bufsize += LINEBUFSIZE;
+        *buf = xrealloc(*buf, bufsize);
+        if (!fgets(&(*buf)[bufsize - LINEBUFSIZE - 1], LINEBUFSIZE + 1, eval))
         {
-            *errstr = xasprintf(_("output of '%s' is longer than %d characters"),
-                    arg, LINEBUFSIZE - 1);
+            *errstr = xasprintf(_("cannot read output of '%s'"), arg);
+            pclose(eval);
             free(*buf);
             *buf = NULL;
             return CONF_EIO;
         }
-        if ((*buf)[l - 1] == '\n')
+        len = strlen(*buf);
+        if (len > 0 && (*buf)[len - 1] == '\n')
         {
-            (*buf)[l - 1] = '\0';
-            if (l - 1 > 0 && (*buf)[l - 2] == '\r')
-            {
-                (*buf)[l - 2] = '\0';
-            }
+            /* Read only the first line. */
+            break;
+        }
+    }
+    while (!feof(eval));
+    pclose(eval);
+
+    if (len > 0 && (*buf)[len - 1] == '\n')
+    {
+        (*buf)[len - 1] = '\0';
+        if (len - 1 > 0 && (*buf)[len - 2] == '\r')
+        {
+            (*buf)[len - 2] = '\0';
         }
     }
 
